// Solutions to the Advent Of Code 2024.
// Copyright (C) 2024  Stefan Mller
//
// This program is free software: you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation, either version 3 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along with
// this program.  If not, see <http://www.gnu.org/licenses/>.

#include <iostream>
#include <sstream>

#include "RedNosedReports.h"
#include <iomanip>

std::string RedNosedReports::getPuzzleName() const
{
    return "Day 2: Red-Nosed Reports";
}

std::string RedNosedReports::getInputFileName() const
{
    return "red-nosed_reports.txt";
}

void RedNosedReports::processDataLine(const std::string& line)
{
    auto isSafe{ true };
    auto slope{ Slope::Unknown };

    std::stringstream stream{ line };
    std::string token;
    std::getline(stream, token, ' ');
    auto prev{ std::stoi(token) };

    while (isSafe && std::getline(stream, token, ' '))
    {
        auto next{ std::stoi(token) };
        auto delta{ next - prev };
        if (delta == 0 || delta > 3 || delta < -3
            || (delta > 0 && slope == Slope::Decreasing)
            || (delta < 0 && slope == Slope::Increasing))
        {
            isSafe = false;
        }
        else
        {
            prev = next;
            if (slope == Slope::Unknown)
            {
                slope = delta > 0 ? Slope::Increasing : Slope::Decreasing;
            }
        }
    }
    if (isSafe)
    {
        part1++;
    }
}

void RedNosedReports::finish()
{
}
